import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

public class MetroNavigator extends MIDlet implements CommandListener  
{ 
    private static MetroNavigator instance = null;

    private Metro   metro;
    private Station from;
    private Station till;
    private int     cmd;
    Route   minTimeRoute;
    Route   minCrossesRoute;

    static final Command QUIT_CMD = new Command(Locale.QUIT, Command.BACK, 1);
    static final Command BACK_CMD = new Command(Locale.BACK, Command.BACK, 1);
    static final Command SELECT_CMD = new Command(Locale.SELECT, Command.ITEM, 1);
    static final Command SEARCH_CMD = new Command(Locale.SEARCH, Command.ITEM, 2);
    static final Command MIN_TIME_CMD = new Command(Locale.MIN_TIME, Command.ITEM, 2);
    static final Command MIN_CROSSES_CMD = new Command(Locale.MIN_CROSSES, Command.ITEM, 2);
    static final Command SCROLL_CMD = new Command(Locale.SCROLL, Command.ITEM, 1);

    private static String[] actionNames =
    {
        Locale.START + "?",
        Locale.END + "?",
        Locale.MIN_TIME + ": ?",
        Locale.MIN_CROSSES + ": ?",
        Locale.SCHEME
    };
    

    List actionList;
    SelectStation selectStation;

    public MetroNavigator() 
    {
        Scheme scheme = new Scheme("/scheme.xml");
        metro = new Metro(scheme.metroLines, scheme.metroCrosses);
        actionList = new List(Locale.ROUTE, List.IMPLICIT, actionNames, null);
	    actionList.setCommandListener(this) ;
	    actionList.addCommand(QUIT_CMD);
        instance = this;
    }

    static MetroNavigator getInstance()
    {
        return(instance);
    }
    
    public void setStation(Station station) 
    { 
        if (cmd == 0) 
        { 
            from = station;
        } 
        else 
        {
            till = station;
        }
        if (from != null && till != null) 
        { 
            minTimeRoute = metro.findRoute(from, till, 0);
            minCrossesRoute = metro.findRoute(from, till, 100);
        }
    }

    protected void destroyApp(boolean unconditional) 
    {
        actionList = null;
    }    

    protected  void pauseApp() 
    {
    }

    protected void startApp() 
    {
        if (from != null) 
        {
            actionList.set(0, Locale.START + from.getShortName(), null);
        }
        if (till != null) 
        { 
            actionList.set(1, Locale.END + till.getShortName(), null);
        }
        if (minTimeRoute != null) 
        { 
            actionList.set(2, Locale.MIN_TIME + ": " + minTimeRoute.time, null);
            actionList.set(3, Locale.MIN_CROSSES + ": " + minCrossesRoute.crosses.length, null);
        }
        Display.getDisplay(this).setCurrent(actionList);
    }

    public void quit() 
    {
	destroyApp(true);
	notifyDestroyed();
	instance = null;
    }

    static final int ERROR_MSG_TIMEOUT = 2000;

    void showAlert(AlertType type, String title, String text, int timeout)
    {
        Alert currentAlert = new Alert(title);
        currentAlert.setTimeout(timeout);
        currentAlert.setString(text);
        currentAlert.setType(type);
        Display disp = Display.getDisplay(this);
        disp.setCurrent(currentAlert, actionList);
    }

    public void showError(String text)
    {
        showAlert(AlertType.ERROR, Locale.ERROR, text, ERROR_MSG_TIMEOUT);
    }

    boolean correctState()
    { 
        if (from == null) 
        {
            showError(Locale.NO_START);
        } 
        else if (till == null) 
        {
            showError(Locale.NO_END);
        } 
        else 
        {
            return true;
        }
        return false;
    }

    void showRoute(Route route) 
    {
        String text = Locale.TOTAL_TIME + route.time
            + Locale.CROSSES + route.crosses.length
            + Locale.STATIONS + route.nStations;
        for (int i = 0; i < route.crosses.length; i++) 
        { 
            String from = route.crosses[i].src.getShortName();
            String till = route.crosses[i].dst.getShortName();
            if (from.equals(till)) 
            { 
                text += "\n" + (i+1) + Locale.CROSS_AT + from
                    + Locale.TO_LINE + route.crosses[i].dst.line.name;
            } 
            else 
            {
                text += "\n" + (i+1) + Locale.CROSS_AT + from
                    + Locale.TO_STATION + till + Locale.OF_LINE + route.crosses[i].dst.line.name;
            }
        } 
        showAlert(AlertType.INFO, 
            Locale.ROUTE, text, Alert.FOREVER);
    }

    public void commandAction(Command c, Displayable d) 
    {
        if (c == QUIT_CMD) 
        { 
            quit();
            return;
        }
        cmd = actionList.getSelectedIndex();
        Route route;
        switch (cmd) 
        { 
            case 0:
            case 1:
                new SelectStation();
                break;
            case 2:
                if (correctState()) 
                {
                    showRoute(minTimeRoute);
                }
                break;
            case 3:
                if (correctState())  
                {
                    showRoute(minCrossesRoute);
                }
                break;
            default:
                new SchemeViewer(this);
        }
    }
}
